#cs ----------------------------------------------------------------------------

 AutoIt Version: 3.3.6.0
 Author:         Burak Keskin

 Script Function:
	Ini-Class.

#ce ----------------------------------------------------------------------------

; Script Start - Add your code below here

#cs
 * Erstellt ein Ini-Objekt.
 *
 * @param  $Pfad	Pfad zur Ini-Datei. (Optional)
 * @return		Gibt ein Ini-Objekt zurck.
#ce
Func _New_IniObject($Pfad = "")
	Local $oIni = _AutoItObject_Create()
	_AutoItObject_AddProperty($oIni, "Array")
	_AutoItObject_AddProperty($oIni, "Pfad")

	_AutoItObject_AddMethod($oIni, "LoadFile", "_LoadFile")
	_AutoItObject_AddMethod($oIni, "Read", "_Read")
	_AutoItObject_AddMethod($oIni, "Reload", "_Reload")
	_AutoItObject_AddMethod($oIni, "ReadPos", "_ReadPos1")
	_AutoItObject_AddMethod($oIni, "GetPosition", "_ReadPos2")
	_AutoItObject_AddMethod($oIni, "Write", "_IniWrite")
	_AutoItObject_AddMethod($oIni, "ReadSection", "_IniReadSection")


	If $Pfad <> "" Then _LoadFile($oIni, $Pfad)

	Return $oIni
EndFunc   ;==>LinkedList

#cs
 * Liest einen Sektor aus und speichert diese in einen Array.
 *
 * @param  $Section	Name des Sektors.
 * @return		Ein Array, das die Keys + Value enthlt.
#ce
Func _IniReadSection($oSelf, $Section)
	Local $CurArray = $oSelf.Array
	Local $Counter
	For $i = 0 To UBound($CurArray) -1
		If $CurArray[$i][0] = $Section Then $Counter += 1
	Next
	Local $NewArray[$Counter][2]
	$Counter = 0
	For $i = 0 To UBound($CurArray) -1
		If $CurArray[$i][0] = $Section Then
			$NewArray[$Counter][0] = $CurArray[$i][1]
			$NewArray[$Counter][1] = $CurArray[$i][2]
			$Counter += 1
		EndIf
	Next
	Return $NewArray
EndFunc

#cs
 * Schreibt einen Wert in die Ini-Datei, sobald dieser nicht vorhanden bzw. gleich ist.
 *
 * @param  y	(Beschreibung des Parameters)
 * @return		(Beschreibung des Rückgabewertes)
#ce
Func _IniWrite($oSelf, $Section, $Key, $Value)
	If $Value <> $oSelf.Read($Section, $Key) Then
		IniWrite($oSelf.Pfad ,$Section, $Key, $Value)
		Return True
	EndIf
	Return False
EndFunc

Func _ReadPos2($oSelf, $Key) ;Extra fr Controls
	Local $Read = _Read($oSelf, "Controls", $Key)
	If $Read = "" Then Return -1
	Local $Pos = StringSplit($Read, ",", 2)
	Return $Pos
EndFunc

Func _ReadPos1($oSelf, $Section, $Key) ;Extra fr GUIs
	Local $Read = _Read($oSelf, $Section, $Key)
	If $Read = "" Then Return -1
	Local $Pos = StringSplit($Read, ",", 2)
	Return $Pos
EndFunc

Func _Reload($oSelf)
	Return _LoadFile($oSelf, $oSelf.Pfad)
EndFunc

;Liest aus einem Array die geforderten Daten heraus.
Func _Read($oSelf, $Section, $Key)
	Local $Array = $oSelf.Array
	For $i = 0 To UBound($Array) - 1
		Switch $Array[$i][0]
			Case $Section
				If $Array[$i][1] = $Key Then Return $Array[$i][2]
		EndSwitch
	Next
	Return SetError(1, 0, 0)
EndFunc   ;==>_IniRead_Array

;Ldt eine INI-Datei
Func _LoadFile($oSelf, $Pfad)
	Local $SectionName = IniReadSectionNames($Pfad)
	If @error Then Exit MsgBox(16, "Critical Error", "Can not open: " & $Pfad)
	Local $Loaded[1][3]
	Local $ReadSection, $int = 0
	For $i = 1 To $SectionName[0]
		$ReadSection = _LoadSection($Pfad, $SectionName[$i])
		ReDim $Loaded[$int + UBound($ReadSection)][3]
		For $j = 0 To UBound($ReadSection) - 1
			$Loaded[$int][0] = $ReadSection[$j][0]
			$Loaded[$int][1] = $ReadSection[$j][1]
			$Loaded[$int][2] = $ReadSection[$j][2]
			$int += 1
		Next
	Next
	$oSelf.Array = $Loaded
	$oSelf.Pfad = $Pfad
	Return $Loaded
EndFunc   ;==>_IniReadToArray

;Ldt eine Section.
Func _LoadSection($Pfad, $SectionName)
	Local $Section = IniReadSection($Pfad, $SectionName)
	Local $Loaded[$Section[0][0]][3]
	For $i = 1 To $Section[0][0]
		$Loaded[$i - 1][0] = $SectionName
		$Loaded[$i - 1][1] = $Section[$i][0]
		$Loaded[$i - 1][2] = $Section[$i][1]
	Next
	Return $Loaded
EndFunc   ;==>_IniReadSectionToArray








